      PROGRAM RNJ22A
C
C     ******************************************************************
C     *                                                               *
C     *                      CELL THICKNESS EVALUATION                *
C     *                                                               *
C     ******************************************************************
C 
C     Version 06/92 
C     BASED ON PROGRAM XXII WRITTEN BY R.N.JONES AND R.P.YOUNG, DIVISION OF
C     CHEMISTRY, NATIONAL RESEARCH COUNCIL OF CANADA, OTTAWA, CANADA.
C 
C     THIS VERSION OF THE PROGRAM HAS BEEN MODIFIED BY C. D. KEEFE TO
C         RUN ON AN IBM COMPATIBLE PC COMPUTER.
C
C     THIS PROGRAM COMPUTES THE CELL PATH LENGTH FROM THE INTERFERENCE 
C         FRINGES OF THE EMPTY CELL.
C
C     IT USES THE SAVITZKY PEAKFINDER PROGRAM TO LOCATE THE POSITIONS 
C         OF THE FRINGE EXTREMA AND DETAILS OF THIS PART OF THE 
C         PROGRAM ARE DISCUSSED IN N.R.C.C. BULLETIN NO. 11. 
C         THE PARTS OF THE PROGRAM CONCERNED SPECIFICALLY WITH THE 
C         CELL THICKNESS EVALUATION ARE DISCUSSED IN BULLETIN NO.13. 
C 
C     THE INPUT DATA MUST BE IN WAVENUMBER UNITS OF CM.-1 AND
C         INTENSITY UNITS OF ABSORBANCE.
C 
C     THIS PROGRAM IS DIMENSIONED FOR 8192 DATA POINTS AND 200 FRINGES.
C
      CHARACTER*12 INFILE,OUTFILE
      CHARACTER*192 COMM
      LOGICAL DOSPC
      PARAMETER (IRD=5, IPRNT=6, NPMAX=8192)
      DIMENSION IX(NPMAX),IY(NPMAX),Y(NPMAX),T(2),XNAME(2),
     $ HEAD1(20),HEAD2(20)
      DATA XNAME/'PEAK','DIP '/
C
      Write(*,*)'Cell Thickness Determination Version 06/92'
      OPEN(IRD,FILE='RNJ22A.ASC',STATUS='OLD',MODE='READ')
      READ(IRD,*)IRUNT
      DO 70 IRUN=1,IRUNT
	 WRITE(*,*)'Reading data from RNJ22A.ASC for run ',irun
	 READ(IRD,401)HEAD1
	 READ(IRD,*)INFILE
	 DOSPC=.FALSE.
	 IPOS=9
	 DO 5 I=9,2,-1
	  IF(INFILE(I:I).EQ.'.')THEN
	    IPOS=I
	    GO TO 6
	  END IF
   5    CONTINUE
   6    IF(INFILE(IPOS:IPOS+3).EQ.'.SPC'.OR.INFILE(IPOS:IPOS+3).EQ.
     $   '.spc')DOSPC=.TRUE.
	 OUTFILE=INFILE(:IPOS-1)//'.OUT'
	 OPEN(IPRNT,FILE=OUTFILE,STATUS='UNKNOWN',MODE='WRITE')
	 WRITE (IPRNT,400)
	 READ (IRD,401) HEAD1 
	 READ (IRD,401) HEAD2 
	 READ (IRD,*) ICNTR,TOL
C 
C       USE ICNTR=1 FOR A NINE POINT INTERPOLATION OF THE BAND MAXIMUN 
C          AND ICNTR=2 FOR A FIFTEEN POINT INTERPOLATION. 
C
C       USE ICNTR=1 FOR SPECTRA WITH SHARP NARROW BANDS AND ICNTR=2 FOR
C          FLATTISH SPECTRA. IF AN ERROR IS INDICATED CHANGE ICNTR AND RERUN.
C          IF THIS FAILS CHECK THE NOISE LEVEL IN THE SPECTRUM.
C 
C       TOL IS THE NOISE REJECTION GATE IN TRANSMITTANCE X1000. 
C          THE RECOMMENDED STANDARD VALUE IS 5.0
C
	 IF(DOSPC)THEN
	  CALL READSC(INFILE,Y,NP,XSP,XEP,XT,YT,RES,COMM,IER)
	 ELSE
	  CALL READAS(INFILE,Y,NP,XSP,XEP)
	 END IF
	 IF(NP.GT.NPMAX)THEN
	  WRITE(*,*)'Input file too large. max. number of points allowed',
     $   NPMAX
	  GO TO 60
	 END IF
	 DELTAX=(XSP-XEP)/(NP-1)
	 DO 10 I=1,NP
	  IX(I)=10*(XSP-DELTAX*(I-1))
	  IY(I)=INT(EXP(2.302585*(3.-Y(I)))+.5)
  10    CONTINUE
	 WRITE (IPRNT,405) HEAD1 
	 WRITE (IPRNT,405) HEAD2
	 WRITE (IPRNT,406) NP,XSP,DELTAX
C 
C 
	 DO 12 I=1,NP 
	   IF (IY(I).LE.999) GO TO 11 
	   XSTOP=IX(I)*0.1 
	   WRITE (IPRNT,409) XSTOP 
	   WRITE (*,409) XSTOP 
	   WRITE (IPRNT,411)(IX(J),IY(J),J=1,NP) 
	   GO TO 60 
   11     CONTINUE 
	   IF (IY(I).GE.30) GO TO 12 
	   XSTOP=IX(I)*0.1 
	   WRITE (IPRNT,410) XSTOP 
	   WRITE (*,410) XSTOP 
	   WRITE (IPRNT,411)(IX(J),IY(J),J=1,NP) 
	   GO TO 60 
   12   CONTINUE 
C
	 WRITE(*,*)'  SEARCHING FOR MAXIMA AND MINIMA.'
	 ISKIP=1
C 
   20   CALL PKFND3 (IX,IY,TOL,AVDT,XNAME,ISKIP,WI,NP,ICNTR,IPRNT)
	 T(ISKIP)=AVDT 
C 
C       AVDT IS THE AVERAGE CALCULATED VALUE OF THE CELL THICKNESS IN CM. 
C 
	 IF (ISKIP.EQ.2) GO TO 50 
	 DO 40 I=1,NP 
   40   IY(I)=1000-IY(I) 
	 ISKIP=2 
	 GO TO 20 
C 
   50   AVT=0.5*(T(1)+T(2)) 
	 AVM=AVT*10000.0 
	 WRITE (IPRNT,407) AVT,AVM
	 WRITE (*,407) AVT,AVM
   60   CLOSE(IPRNT)
   70 CONTINUE
      CLOSE(IRD)
      STOP
C 
  400 FORMAT (2X,'RNJ22A    CELL THICKNESS EVALUATION.'//)
  401 FORMAT (20A4)
  405 FORMAT (2X,20A4)
  406 FORMAT (//2X,2HNP,I8,15X,2HWB,F9.1,15X,2HWI,F6.1//)
  407 FORMAT (//2X,'THE AVERAGE CELL THICKNESS = ',F7.6,' CM  (OR ',
     $F7.2,' MICROMETRES)')
  408 FORMAT (//2X,'COMPUTATION TERMINATED, TIME LIMIT EXCEEDED')
  409 FORMAT (//2X,'COMPUTATION TERMINATED, POINT ON OR BELOW BASELINE
     $AT ',F6.1,' CM.-1'/)
  410 FORMAT (//2X,'COMPUTATION TERMINATED, TRANSMISSION LESS THAN 3 PE
     $RCENT AT ',F6.1,' CM.-1'/)
  411 FORMAT (11(2X,I5,I4)) 
      END 
C
C 
C     ******************************************************************
C 
      SUBROUTINE PKFND3 (IX,IY,TOL,AVDT,XNAME,ISKIP,WI,NP,ICNTR,IPRNT) 
C 
      DIMENSION IX(*),IY(*),NPOINT(25),NDERV(25),PKNU(200),PKTABL(1000),
     $ XNAME(*)
C 
C     ***  INITIALIZATION OF PEAKFIND AND EVALUATION OF DERIVATIVE *** 
C
   20 NPEAK=1 
      K=12 
      JA=26 
      JB=-13 
      DO 21 I=1,25 
      NPOINT(I)=IY(I) 
      JB=JB+1 
   21 NDERV(I)=NPOINT(I)*JB 
      NTP=0 
      DO 22 I=1,25 
   22 NTP=NDERV(I)+NTP 
      IF (NTP.GT.0) GO TO 24 
      LPV=1 
      TEST=NPOINT(13) 
      GO TO 25 
   24 LPV=2 
   25 DO 100 J=JA,NP 
      K=K+1 
      NTP=0 
      NEXPT=IY(J) 
      DO 26 I=1,24 
      NI=I+1 
      NTEMPA=NPOINT(NI) 
      NPOINT(I)=NTEMPA 
   26 NDERV(I)=NDERV(NI)-NTEMPA 
      NPOINT(25)=NEXPT 
      NDERV(25)=NEXPT*12 
C 
C     THE SEARCH FOR THE PEAK IS MADE FROM THE ELEVEN CENTRAL POINTS. 
C 
      DO 35 I=8,18 
   35 NTP=NTP+NDERV(I) 
   43 IF (LPV.EQ.1) GO TO 38 
      IF (NTP.GT.0) GO TO 100 
      LPV=1 
      TEST = NPOINT(13) 
      GO TO 100 
   38 IF (NTP.LT.0) GO TO 100 
      LPV=2 
      TRANS=NPOINT(13) 
      DENS=3.0-ALOG10(TRANS) 
      XX=0.0 
      IF ((TEST-TRANS).LE.TOL) GO TO 100 
C 
C     *********  INTERPOLATION OF PEAK POSITION AND INTENSITY  *********
C 
   45 CALL CENTR2 (NPOINT,XX,YY,TRANS,DENS,ICNTR,IPRNT) 
      JJ=J-12 
      NUPK=IX(JJ) 
      ANUPK=0.1*NUPK-XX*WI 
      PKNU(NPEAK)=ANUPK 
      IF (ISKIP.EQ.2) GO TO 48 
   47 TRANS=TRANS*0.1 
      NAME=1 
      GO TO 49 
   48 TRANS=1000-TRANS 
      DENS=3.0-ALOG10(TRANS) 
      TRANS=TRANS*0.1 
      NAME=2 
C 
   49 CALL PKSTR2 (NPEAK,ANUPK,TRANS,DENS,PKTABL,XNAME,NAME) 
C 
  100 CONTINUE 
C 
C     *********  CALCULATION OF THE CELL THICKNESS  ********** 
C 
      CALL THICK (PKNU,NPEAK,PKTABL,TOL,AVDT,ISKIP,ICNTR,IPRNT) 
C 
      RETURN 
      END 
C 
C     ******************************************************************
C 
      SUBROUTINE CENTR2 (NPOINT,XX,YY,TRANS,DENS,ICNTR,IPRNT) 
C 
      DIMENSION NPOINT(*),TRN(15),P(7),PM(7)
      LOGICAL TESTA,TESTB 
      IF (ICNTR.EQ.2) GO TO 40 
C 
C     **********  INTERPOLATION WITH A NINE POINT CUBIC CURVE  *********
C 
   10 DO 11 I=1,9 
      IP=I+8 
   11 TRN(I)=NPOINT(IP) 
      DO 12 I=1,4 
      IP=10-I 
      P(I)=TRN(I)+TRN(IP) 
   12 PM(I)=TRN(I)-TRN(IP) 
      A30=-21.0*P(1)+14.0*P(2)+39.0*P(3)+54.0*P(4)+59.0*TRN(5) 
      A30=A30/231.0 
      A31=86.0*PM(1)-142.0*PM(2)-193.0*PM(3)-126.0*PM(4) 
      A31=A31/1188.0 
      A32=28.0*P(1)+7.0*P(2)-8.0*P(3)-17.0*P(4)-20.0*TRN(5) 
      A32=A32/924.0 
      A33=-14.0*PM(1)+7.0*PM(2)+13.0*PM(3)+9.0*PM(4) 
      A33=A33/1188.0 
      GO TO 50 
C 
C     ********  INTERPOLATION WITH A FIFTEEN POINT CUBIC CURVE  ********
C 
   40 DO 41 I=1,15 
      IP=I+5 
   41 TRN(I)=NPOINT(IP) 
      DO 42 I=1,7 
      IP=16-I 
      P(I)=TRN(I)+TRN(IP) 
   42 PM(I)=TRN(I)-TRN(IP) 
      A30=-78.0*P(1)-13.0*P(2)+42.0*P(3)+87.0*P(4)+122.0*P(5)+147.0*P(6)
     $ +162.0*P(7)+167.0*TRN(8)
      A30=A30/1105.0 
      A31=12922.0*PM(1)-4121.0*PM(2)-14150.0*PM(3)-18334.0*PM(4)-17842.0
     $ *PM(5)-13843.0*PM(6)-7506.0*PM(7)
      A31=A31/334152.0 
      A32=91.0*P(1)+52.0*P(2)+19.0*P(3)-8.0*P(4)-29.0*P(5)-44.0*P(6)- 
     $ 53.0*P(7)-56.0*TRN(8)
      A32=A32/12376.0 
      A33=-91.0*PM(1)-13.0*PM(2)+35.0*PM(3)+58.0*PM(4)+61.0*PM(5)+ 
     $ 49.0*PM(6)+27.0*PM(7)
      A33=A33/47736.0 
C 
   50 NUM=1 
      IF (ABS(A33).GT.10E-10) GO TO 51 
      XX=-A31/(2.0*A32) 
      YY=A30+XX*(A31+XX*A32) 
      WRITE (IPRNT,474) NUM 
      GO TO 100 
   51 DENOM=3.0*A33 
      DISC=A32*A32-A31*DENOM 
      IF (DISC) 54,55,56 
   54 TRANS=NPOINT(13) 
      DENS=3.0-ALOG10(TRANS) 
      WRITE (IPRNT,475) NUM 
      XX=0.0 
      YY=TRANS 
      GO TO 101 
   55 XX=-A32/DENOM 
      YY=A30+XX*(A31+XX*(A32+XX*A33)) 
      GO TO 100 
   56 RAD=SQRT(DISC) 
      XMINA=(-A32+RAD)/DENOM 
      XMINB=(-A32-RAD)/DENOM 
      ORDA=A30+XMINA*(A31+XMINA*(A32+XMINA*A33)) 
      ORDB=A30+XMINB*(A31+XMINB*(A32+XMINB*A33)) 
      TESTA=ABS(XMINA).GT.4.0 
      TESTB=ABS(XMINB).GT.4.0 
      IF (TESTA.AND.TESTB) GO TO 60 
      IF (TESTA) GO TO 61 
      IF (ORDA.GT.ORDB) GO TO 62 
   65 XX=XMINA 
      YY=ORDA 
      GO TO 100 
   60 XX=0.0 
      YY=NPOINT(13) 
      GO TO 100 
   62 IF (TESTB) GO TO 65 
   61 XX=XMINB 
      YY=ORDB 
  100 TRANS=YY 
      DENS=3.0-ALOG10(TRANS) 
  101 NUM=NUM+1 
      RETURN 
  470 FORMAT (2X,'QUADRATIC,    XX=',1PE14.6,'     YY=',1PE14.6)
  474 FORMAT (2X,'QUADRATIC INTERPOLATION USED FOR NO..',I3)
  475 FORMAT (2X,'BOTH ROOTS FOR BAND N0.',I3,' ARE IMAGINARY'/
     $ '-- THE INPUT VALUES ARE LISTED IN THE TABLE.')
      END 
C 
C     ******************************************************************
C 
      SUBROUTINE PKSTR2 (NPEAK,ANUPK,TRANS,DENS,PKTABL,XNAME,NAME) 
C 
      DIMENSION PKTABL(*),XNAME(*)
C 
      K=4*NPEAK-3 
      PKTABL(K)=ANUPK 
      PKTABL(K+1)=TRANS 
      PKTABL(K+2)=DENS 
      PKTABL(K+3)=XNAME(NAME) 
      NPEAK=NPEAK+1 
      RETURN 
      END 
C 
C     ******************************************************************
C 
      SUBROUTINE THICK (XF,NF,PKTABL,TOL,AVDT,ISKIP,ICNTR,IPRNT) 
C 
      DIMENSION XF(200),PKTABL(*),DX(100),DT(100),ORDER(200)
C 
C     NF IS THE NUMBER OF FRINGES. 
C     XF IS THE WAVENUMBER OF THE FRINGE EXTREMUM. 
C 
      NF=NF-1 
      N=NF/2 
      NJ=FLOAT(NF)/2.0+0.5 
      SDX=0.0 
      SDT=0.0 
      DO 10 I=1,NJ 
      J=I+N 
      DX(I)=XF(I)-XF(J) 
      DT(I)=0.5*N/(DX(I)*1.00026) 
      SDX=SDX+DX(I) 
   10 SDT=SDT+DT(I) 
      AVDX=SDX/NJ 
      AVDT=SDT/NJ 
C 
      SX=0.0 
      ST=0.0 
      DO 20 I=1,NJ 
      DELX=DX(I)-AVDX 
      SX=SX+DELX*DELX 
      DELT=DT(I)-AVDT 
   20 ST=ST+DELT*DELT 
      SX=SQRT(SX/NJ) 
      ST=SQRT(ST/NJ) 
C 
      DO 21 I=1,NF 
   21 ORDER(I)=2.00052*AVDT*XF(I) 
C 
      CALL PKOUT2 (NF,PKTABL,ORDER,TOL,ISKIP,ICNTR,IPRNT) 
C 
      IF (ISKIP.EQ.2) GO TO 25 
      WRITE (IPRNT,420)
      WRITE (*,420)
      GO TO 30 
   25 WRITE (IPRNT,421)
      WRITE (*,421)
   30 WRITE (IPRNT,422) 
      DO 40 I=1,NJ 
      J=I+N 
   40 WRITE (IPRNT,423) I,J,DX(I),DT(I) 
      WRITE (IPRNT,424) N,AVDX,SX,AVDT,ST
      WRITE (*,424) N,AVDX,SX,AVDT,ST
C 
  420 FORMAT (2X,'------------ CELL THICKNESS CALCULATED FROM TRANSMI',
     $'SSION MINIMA ------------')
  421 FORMAT (2X,'------------ CELL THICKNESS CALCULATED FROM TRANSMI',
     $'SSION MAXIMA ------------')
  422 FORMAT (14X,'FRINGE NOS.',5X,'SPACING(CM.-1)',3X,'CELL THICKNESS(C
     $M)'/)
  423 FORMAT (14X,I3,' - ',I3,8X,F7.2,12X,F8.6)
  424 FORMAT (2X,'THE AVERAGE SPACING OF ',I3,' FRINGES IS ',F7.2,
     $' CM-1',' STANDARD DEVIATION',F7.2/10X,'THE AVERAGE CELL ',
     $'THICKNESS IS ',F7.6,' CM',2X,'STANDARD DEVIATION ',F7.6)
      RETURN 
      END 
C 
C     ******************************************************************
C 
      SUBROUTINE PKOUT2 (NPEAK,PKTABL,ORDER,TOL,ISKIP,ICNTR,IPRNT) 
C 
      DIMENSION PKTABL(*),ORDER(*)
C 
      GO TO (15,16),ISKIP 
   15 WRITE (IPRNT,481) 
      TOL=0.1*TOL 
      GO TO 17 
   16 WRITE (IPRNT,482) 
   17 WRITE (IPRNT,483) 
      DO 20 K=1,NPEAK 
      J=K*4 
      KJ=J-3 
   20 WRITE (IPRNT,484) K,(PKTABL(I),I=KJ,J),ORDER(K) 
      IF (ICNTR.EQ.2) GO TO 21 
      WRITE (IPRNT,485) TOL 
      RETURN 
   21 WRITE (IPRNT,486) TOL 
      RETURN 
C 
  481 FORMAT (//2X,'TRANSMISSION MINIMA.'/)
  482 FORMAT (//2X,'TRANSMISSION MAXIMA.'/)
  483 FORMAT (7X,'NO.    POSITION    PERCENT TRANS.   ABSORBANCE',10X,
     $'FRINGE ORDER'/)
  484 FORMAT (7X,I3,3X,F9.2,8X,F4.1,10X,F6.4,5X,A4,6X,F6.2)
  485 FORMAT (/' INTERPOLATED WITH A NINE POINT CURVE',5X, 
     $'NOISE REJECTION GATE =',F4.1,' PERCENT'////)
  486 FORMAT (/' INTERPOLATED WITH A FIFTEEN POINT CURVE',5X, 
     $'NOISE REJECTION GATE =',F4.1,' PERCENT'////)
      END 
