CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     SUBROUTINE KKTSD                                                 C
C                                                               C
C     --- THIS ROUTINE CALCULATE THE N SPECTRUM FROM THE K SPECTRUM.   C
C        IT USES THE SUBTRACTIVE OR STANDARD KRAMERS-KRONIG ALGORITHM.        C
C        FOR DETAILS SEE APPENDIX TO PAPER BY HAWRANEK          C
C        AND JONES (SPECTROCHIMICA ACTA 32A, 99 (1976))         C
C                                                               C
C        KKTSD       --  FULL KK TRANSFORMATION WITHOUT APPROXIMATION.        C
C                 INTEGRATION OVER THE WHOLE RANGE OF SPECTRA          C
C                                                               C
C     RESTRICTIONS:  A SPECTRUM WITH A CONSTANT WAVENUMBER ENCODING    C
C                  INTERVAL IS NECESSARY.                       C
C                                                               C
C      THIS ROUTINE IS BASICALLY THE SAME AS KKT WRITTEN               C
C      BY S.L.ZHANG, (AUGUST/1990) WITH THE FOLLOWING CHANGES MADE BY  C
C      C. D. KEEFE FOR RNJ46A      (SEPT/1990):                        C
C         - NAME CHANGES TO BE CONSISTENT WITH NAMES IN REST OF RNJ46A  C
C            X2 TO XS, XK TO XCAY, RK TO CAY, RN TO EN, N TO NP,       C
C            H TO DELX.                                         C
C         - CALLS FOR TIMING REMOVED                                   C
C         - ARRAYS AND VARIABLES PLACED IN COMMOM INSTEAD OF CALL      C
C         - DEFINING LOOPS FOR X AND XS REMOVED SINCE THEY WERE        C
C           ALREADY CALCULATED BEFORE THE CALL (I.E. RNJ46A NEEDS THEM).C
C         - NEW CODE TO ADD THE OPTION OF A SUBTRACTIVE KK.            C
C       CHANGES MADE APRIL 1991                                 C
C         - KK CAHNGED TO USE MACLAURIN'S FORMULA AS IN OHTA AND ISHIDA C
C           APPLIED SPECTROSCOPY, 42,952, (1988)                C
C                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE KKTSD
      INCLUDE 'RNJ46A.PAR'
      REAL*8 XS
      DIMENSION XCAY(NPMAX)
      COMMON/CKK/X(NPMAX),XS(NPMAX),CAY(NPMAX),EN(NPMAX),ENR,WR,DELX,
     $ NP, MODE
C
      HPI=0.63661977236758D0*DELX
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     ORIGINAL KKT HAD THE FOLLOWING LOOP TO DEFINE X AND XS LOOP       C
C     X(1)=XSP                                                          C
C     XS(1)=XSP*XSP                                              C
C     XCAY(1)=XSP*CAY(1)                                         C
C     DO 10 I=2,N                                                C
C     X(I)=X(I-1)+DELX                                                  C
C     XS(I)=X(I)*X(I)                                                   C
C 10  XCAY(I)=X(I)*CAY(I)                                        C
C                                                                C
C     BUT IT WAS REDUCED TO THE FOLLOWING LOOP SINCE EVERYTHING EXCEPT         C
C     XCAY WAS ALREADY DEFINED BEFORE THE CALL.                  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      DO 10 I=1,NP
	XCAY(I)=X(I)*CAY(I)
  10  CONTINUE
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     THE FOLLOWING WAS ADDED TO ALLOW FOR SUBTRACTIVE KK.              C
      C=0.0
      IF(MODE.EQ.1)THEN
	X2=WR*WR
	D=0.5*DELX
	DO 15 J=1,NP
	  DX=ABS(X(J)-WR)
	  IF(DX.GT.D)C=C+XCAY(J)/(XS(J)-X2)
   15  CONTINUE
      END IF
      C=ENR-HPI*C
C     FOR MODE=2 C=ENR
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      HPI=2.*HPI
      DO 20 I=2,NP,2
	SUM=0.
	DO J=1,NP,2
	  SUM=SUM+XCAY(J)/(XS(J)-XS(I))
	END DO
	EN(I)=HPI*SUM+C
20    CONTINUE
      DO 30 I=1,NP,2
	SUM=0.
	DO J=2,NP,2
	  SUM=SUM+XCAY(J)/(XS(J)-XS(I))
	END DO
	EN(I)=HPI*SUM+C
30    CONTINUE
      RETURN
      END
